const LightColors = {
    white: 'ffffff',
    yellow: 'ffff00',
    blue: '0000ff',
    green: '00ff00'
}

const markerGeo = new THREE.SphereGeometry(8, 4, 4);
const markerMat = new THREE.MeshBasicMaterial({
    color: 0xffff00,
    wireframe: true,
    side: THREE.DoubleSide,
});

class Light {
    constructor(id, scene, uiContainer) {
        Object.assign(this, {
            id,
            scene,
            uiContainer,
            distance: 192,
            height: 64,
        });

        this.spotLight = new THREE.SpotLight(0xffffff);
        this.spotLight.intensity = 0.5;
        this.spotLight.target = this.scene;

        this.marker = new THREE.Mesh(markerGeo, markerMat);

        this.updatePosition();

        scene.add(this.spotLight);
        scene.add(this.marker);

        this.uiPanel = document.createElement('div');
        this.uiPanel.classList.add('lightui');
        Object.values(LightColors).forEach(color => {
            let colorButton = document.createElement('button');
            colorButton.style.backgroundColor = `#${color}`;
            colorButton.innerHTML = `&nbsp;`;
            this.uiPanel.append(colorButton);
            colorButton.addEventListener('click', e => {
                this.setColor(color);
            });
        });

        let addRange = (min, max, propertyName) => {
            let range = document.createElement('input');
            range.setAttribute('type', 'range');
            range.setAttribute('min', min);
            range.setAttribute('max', max);
            range.value = this[propertyName];
            this.uiPanel.append(range);
            range.addEventListener('input', e => {
                this[propertyName] = parseInt(range.value);
                this.updatePosition();
            })
        }

        addRange(0, 256, 'distance');
        addRange(0, 128, 'height');

        this.uiContainer.append(this.uiPanel);
    }

    setColor(color) {
        this.spotLight.color.set(parseInt(color, 16));
    }

    updatePosition() {
        let position = [
            this.distance * Math.sin(this.id * (Math.PI / 3)),
            this.height,
            this.distance * Math.cos(this.id * (Math.PI / 3))
        ];
        this.spotLight.position.set(...position);
        this.marker.position.set(...position);
    }
}